/*
 * This file is part of the Cforall project
 *
 * $Id: TupleExpr.cc,v 1.11 2005/08/29 20:59:26 rcbilson Exp $
 *
 */

#include "Expression.h"
#include "utility.h"


TupleExpr::TupleExpr( Expression *_aname ) : Expression( _aname )
{
}

TupleExpr::TupleExpr( const TupleExpr &other )
    : Expression( other )
{
    cloneAll( other.exprs, exprs );
}

TupleExpr::~TupleExpr()
{
    deleteAll( exprs );
}

void 
TupleExpr::print( std::ostream &os, int indent ) const
{
    os << std::string( indent, ' ' ) << "Tuple:" << std::endl;
    printAll( exprs, os, indent+2 );
    Expression::print( os, indent );
}

SolvedTupleExpr::SolvedTupleExpr( std::list<Expression *> &_exprs, Expression *_aname )
    :  Expression( _aname )
{
    std::copy(_exprs.begin(), _exprs.end(), back_inserter(exprs));
}

SolvedTupleExpr::SolvedTupleExpr( const SolvedTupleExpr &other )
    : Expression( other )
{
    cloneAll( other.exprs, exprs );
}

void
SolvedTupleExpr::print( std::ostream &os, int indent ) const
{
    os << std::string( indent, ' ' ) << "Solved Tuple:" << std::endl;
    printAll( exprs, os, indent+2 );
    Expression::print( os, indent );
}

